#!/usr/bin/python
# coding: UTF-8

# import
import os,sys,re,MeCab,logging
import numpy as np
import pandas as pd
from bs4 import BeautifulSoup as bsp
from gensim.models import word2vec



# mecab wakati module
def mecab_wakati(text):
    tagger = MeCab.Tagger("-Owakati")
    wakati = tagger.parse(text)
    return wakati

# mecab list module
def mecab_list(text):
    tagger = MeCab.Tagger("-Ochasen")
    tagger.parse('')
    node = tagger.parseToNode(text)
    word_class = []
    while node:
        word = node.surface
        wclass = node.feature.split(',')
        if wclass[0]:
            if wclass[6] == None:
                word_class.append((word,wclass[0],wclass[1],wclass[2],""))
            else:
                word_class.append((word,wclass[0],wclass[1],wclass[2],wclass[6]))
        node = node.next
    return word_class

# text format
def format_text(text):
    dict = [r'[!-~]',r'[︰-＠]','\n']
    for i in range(len(dict)):
        text=re.sub(dict[i], "", text)
    return text

def wiki_scraping(data):
    scrape_data = []
    soup = bsp(data, "html5lib")
    texts = soup.find_all("text")
    titles = soup.find_all("title")
    for i in range(len(texts)):
        scrape_data.append((mecab_list(format_text(str(titles[i]))),mecab_list(format_text(str(texts[i])))))
    return scrape_data

#word2vec model create module
def word2vec_model(wakati_words, save_path):
    logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)
    sentences = word2vec.Text8Corpus(wakati_words)
    model = word2vec.Word2Vec(sentences, size=200, min_count=20, window=15)
    model.save(save_path)
    print("Succesfully model saved in" + save_path)

#word2vec model to synonym module
def word2vec_synonym(word, model_path):
    model = word2vec.Word2Vec.load(model_path)
    results = model.wv.most_similar(positive=[word])
    return results

"""
#debug
word_test = "水を入れる"
print(mecab_wakati(word_test))
word_test = "水を入れる"
print(mecab_list(word_test))
for result in results:
   print(result)
"""

if __name__ == "__main__":
    with open("wiki.xml") as f:
        s = f.read()
    datas = wiki_scraping(s)
    titles = []
    descr1s = []
    for data in datas:
        titles.append(data[0])
        descr1s.append(data[1])


    #Schema reset
    matter = []
    subject = []
    behavior = []
    jp_particle = []


    #descr2s_extract_SPV
    for i in range(len(descr1s)):
        su = []
        be = []
        pa = []

        #descr1s
        for j in range(len(descr1s[i])):
            if re.match(u"名詞",descr1s[i][j][1]):
                D = j + 1
                if D < len(descr1s[i]) and re.match(u"助詞",descr1s[i][D][1]):
                    E = D + 1
                    if E < len(descr1s[i]) and re.match(u"動詞",descr1s[i][E][1]):
                        su.append(descr1s[i][j][0])#名詞
                        pa.append(descr1s[i][D][0])#助詞
                        be.append(descr1s[i][E][0])#動詞
        for j in range(len(descr1s[i])):
            if re.match(u"名詞",descr1s[i][j][1]):
                D = j + 1
                if D < len(descr1s[i]) and re.match(u"動詞",descr1s[i][D][1]):
                    su.append(descr1s[i][j][0])#名詞
                    be.append(descr1s[i][D][0])#動詞
                    pa.append(None)#助詞

        #descr1s&descr2s append
        subject.append(su)#名詞
        behavior.append(be)#動詞
        jp_particle.append(pa)#助詞


    for i in range(len(titles)):
        ma = []
        for j in range(len(titles[i])):
            if re.match(u"名詞",titles[i][j][1]):
                ma.append(titles[i][j][0])
        matter.append(ma)

    path = 'solr_index.json'
    f = open(path, mode='w')

    index = []
    count_index = 0
    for i in range(len(matter)):
        for j in range(len(subject[i])):
            if matter[i] != [] and matter[i] != ['アップロード', 'ログ', '年月']:
                count_index += 1
                if count_index == 1:
                    f.write('[\n')
                if count_index > 1:
                    f.write(',' + '\n')
                f.write(
                '{' + '\n' +
                '\t' + '\"id\":\"' + str(count_index) + '\",' + '\n' +
                '\t' + '\"matter\":' + str(matter[i]) + ',' + '\n' +
                '\t' + '\"subject\":\"' + str(subject[i][j]) + '\",' + '\n' +
                '\t' + '\"jp_particle\":\"' + str(jp_particle[i][j]) + '\",' + '\n' +
                '\t' + '\"behavior\":\"' + str(behavior[i][j]) + '\"' + '\n' +
                '}'
                )
                index.append((matter[i],subject[i][j],behavior[i][j],jp_particle[i][j]))
    f.write('\n]')
    f.close()

    df = pd.DataFrame(index,columns=['matter', 'subject', 'behavior','jp_particle'])
    df.to_csv("index2.csv")
